/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { memo, useCallback, useEffect, useState } from 'react'
import { ScrollView, StyleSheet, Text, View } from 'react-native'
import { useShiki } from './SyntaxContext'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG)
    console.log('[SyntaxHighlighter]', ...args)
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

interface SyntaxHighlighterProps {
  code: string
  language: string
  theme?: string | ThemeRegistrationAny
  fontSize?: number
}

export const SyntaxHighlighter = memo(({ code, language, theme = 'github-dark', fontSize = 14 }: SyntaxHighlighterProps) => {
  const { highlighter, isReady } = useShiki()
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string | null>(null)

  const highlight = useCallback(async () => {
    if (!highlighter || !isReady)
      return

    try {
      const loadedLangs = highlighter.getLoadedLanguages()

      log('Loaded languages:', loadedLangs)

      // Add options to ensure proper tokenization
      const results = highlighter.codeToTokens(code, {
        lang: language,
        theme,
        includeExplanation: true,
      })

      log('Raw tokenization result:', results)

      // Process tokens to ensure proper granularity
      const processedTokens = results.tokens.map((line) => {
        // Split any tokens that contain multiple semantic parts
        return line.flatMap((token) => {
          const parts = token.content.split(/([{}().,;=+\-*/<>])/g)
          if (parts.length <= 1)
            return [token]

          return parts.filter(Boolean).map(part => ({
            ...token,
            content: part,
            // Preserve the original token's styling
            color: token.color,
            fontStyle: token.fontStyle,
            explanation: token.explanation,
          }))
        })
      })

      log('Processed tokens:', {
        lines: processedTokens.length,
        tokensPerLine: processedTokens.map(line => line.length),
        firstLine: processedTokens[0]?.map(t => ({
          content: t.content,
          color: t.color,
          scopes: t.explanation?.[0]?.scopes,
        })),
      })

      setTokens(processedTokens)
      setError(null)
    }
    catch (err) {
      console.error('Highlighting error:', err)
      setError(err instanceof Error ? err.message : 'Failed to highlight code')
    }
  }, [code, language, theme, highlighter, isReady])

  useEffect(() => {
    highlight()
  }, [highlight])

  useEffect(() => {
    if (tokens.length > 0) {
      log('Rendering tokens:', {
        lineCount: tokens.length,
        tokensPerLine: tokens.map(line => line.length),
        firstLine: tokens[0]?.map(t => ({
          content: t.content,
          color: t.color,
          scopes: t.explanation?.[0]?.scopes,
        })),
      })
    }
  }, [tokens])

  if (!isReady) {
    return (
      <View style={styles.container}>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  if (error) {
    return (
      <View style={styles.container}>
        <Text style={{ color: 'red' }}>{error}</Text>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  return (
    <ScrollView horizontal style={styles.container} contentContainerStyle={{ padding: 10 }}>
      <View>
        {tokens.map((line, i) => (
          <View key={i} style={styles.line}>
            {line.map((token, j) => {
              if (i === 0 && j < 3) {
                log('Rendering token:', {
                  content: token.content,
                  color: token.color,
                  scopes: token.explanation?.[0]?.scopes,
                })
              }

              return (
                <Text
                  key={`${i}-${j}`}
                  style={{
                    color: token.color || '#e1e4e8',
                    fontSize,
                    ...(token.fontStyle && token.fontStyle & 1 && { fontStyle: 'italic' }),
                    ...(token.fontStyle && token.fontStyle & 2 && { fontWeight: 'bold' }),
                    backgroundColor: token.bgColor,
                  }}
                >
                  {token.content}
                </Text>
              )
            })}
          </View>
        ))}
      </View>
    </ScrollView>
  )
})
